Attribute VB_Name = "modMain"
Option Explicit

' Project WB3 created on 17.04.2011 by Alfred Ruf (Nautilo)

' Modul modMain (Startmodul)
' Alfred Ruf 04.05.2011


' alle Kategorien
Private m_CategoryColl As New Collection

' Startkategorie: (a) Kategorie "Zuhause" mit einzigem Knoten "HomeSweetHome"   (Standard-Szenario)
'                 (b) beliebige Kategorie mit mehreren Knoten                   (allgemeiner Ansatz)
Private m_CATH As Category

' Kategorien, welche ausgehend von der Startkategorie in entsprechender Reihenfolge zu besuchen sind
Private m_SortedCategoryColl As New Collection

' Knoten der vorangehenden Kategorie (bezglich aktueller Kategorie)
Public GL_PrevNodeColl As New Collection

' alle Knoten
Private m_AllNodeColl As New Collection

' zur Messung der Rechenzeit
Public STW As StopWatch

' Flag fr Fehlermeldung
Public ErrorRaised As Boolean

' hoher Startwert fr Suche von Minima
Public Const c_MaxSingle As Single = 3.402823E+38

' Dateinamen
Private m_sFileTask As String
Private m_sFileStations As String
Private m_sFileOutput As String
Public Sub Main()
    Dim CAT As Category
    Dim Stris() As String
    
    On Error GoTo ErrorHandler
    
    Stris = Split(Command, " ")
    If UBound(Stris) = 2 Then
        m_sFileTask = Stris(0)
        m_sFileStations = Stris(1)
        m_sFileOutput = Stris(2)
    Else
        RaiseError "Kommandozeilenargumente in folgener Form erwartet:" & vbNewLine & _
                   "        Dateiname1 Dateiname2 Dateiname3" & vbNewLine & _
                   "wobei:" & vbNewLine & _
                   "        Dateiname1: Input fr Reihenfolge" & vbNewLine & _
                   "        Dateiname2: Input fr Stationen" & vbNewLine & _
                   "        Dateiname3: Output fr zu ermittelnden Pfad" & vbNewLine & _
                   "Dateien liegen im Projektordner." & vbNewLine & _
                   "Leerzeichen innerhalb Dateinamen sind unzulssig."
        Exit Sub
    End If
    
    Reset
    
    Set STW = New StopWatch
    STW.Start
    
    If InitStations Then
        If InitTask Then
                    
            Set GL_PrevNodeColl = m_CATH.NodeColl
            
            For Each CAT In m_SortedCategoryColl
                ' CAT: aktuelle Kategorie
                CAT.SearchShortestPaths
            Next
        End If
    End If
    
    Reset
    
    Exit Sub
ErrorHandler:
    RaiseError "Unerwarteter Fehler:" & Err.Description
End Sub
Private Sub Reset()
    ' Collections zurcksetzen
    Dim CAT As Category
    
    For Each CAT In m_CategoryColl
        CAT.Term
    Next
    
    Set m_CategoryColl = Nothing
    Set m_SortedCategoryColl = Nothing
    
    Set m_AllNodeColl = Nothing
    
    Set GL_PrevNodeColl = Nothing
    
    ErrorRaised = False
End Sub
Private Function InitStations() As Boolean
    
    ' Aufarbeitung aller Stationen (Nodes)
    Dim Lines() As String
    
    Dim i As Long
    Dim Pos As Integer
    Dim sCat As String
    Dim CAT As Category
    Dim Valid As Boolean
    Dim sStations As String
    Dim Handled As Boolean
    
    If ReadUTF8File(m_sFileStations, sStations) Then
        Lines = Split(sStations, vbCrLf)
        For i = 0 To UBound(Lines)
            Lines(i) = Trim$(Lines(i))
            If Lines(i) = "" Then
                ' Leerzeile ignorieren
            Else
                Valid = False
                
                ' [Category]Node->(XPos|YPos)
                
                If Left$(Lines(i), 1) = "[" Then
                    Pos = InStr(2, Lines(i), "]")
                    If 0 < Pos Then
                        sCat = Trim$(Mid$(Lines(i), 2, Pos - 2))
                        If sCat = "" Then
                            RaiseError "Bezeichner fr Kategorie erwartet:" & vbNewLine & Lines(i)
                        Else
                            Set CAT = CategoryItem(sCat)
                            If CAT Is Nothing Then
                                Set CAT = New Category
                                CAT.Init sCat
                                m_CategoryColl.Add CAT, CAT.Name
                            End If
                            Valid = CAT.AddNode(Mid$(Lines(i), Pos + 1))
                        End If
                    End If
                End If
                If Not Valid Then
                    If ErrorRaised Then
                        ' spezifische Fehlermeldung ist bereits angezeigt worden
                    Else
                        RaiseError "Fehlerhafte Definition von Station: " & vbNewLine & Lines(i)
                    End If
                    Exit Function
                End If
            End If
        Next
        InitStations = True
        
        ' wird bei Berechnung nicht verwendet -> freigeben
        Set m_AllNodeColl = Nothing
    End If
End Function
Private Function InitTask() As Boolean
    
    ' Aufarbeitung der in festgelegter Reihenfolge zu besuchenden Kategorien
    
    Dim Stris() As String
    Dim i As Integer
    Dim CAT As Category
    Dim UB As Long
    Dim sTask As String
    
    If ReadUTF8File(m_sFileTask, sTask) Then
        
        Stris = Split(sTask, "->")
        
        UB = UBound(Stris)
        
        If Trim$(Stris(UBound(Stris))) = "" Then
            ' Pfeil am Ende: ignorieren
            UB = UB - 1
        End If
        
        If 2 <= UB Then
            For i = 0 To UB
                Stris(i) = Trim$(Stris(i))
                If Stris(i) = "" Then
                    RaiseError m_sFileTask & ": Fehlerhafte Reihenfolge"
                    Exit Function
                End If
                Set CAT = CategoryItem(Stris(i))
                If CAT Is Nothing Then
                    RaiseError m_sFileTask & ": Kategorie """ & Stris(i) & """ hat keine Knoten"
                    Exit Function
                Else
                    If i = 0 Then
                        Set m_CATH = CAT
                        m_CATH.IsStartCategory = True
                    ElseIf SortedCategoryItem(Stris(i)) Is Nothing Then
                        m_SortedCategoryColl.Add CAT, Stris(i)
                        If i = UB Then
                            CAT.IsEndCategory = True
                            If CAT Is m_CATH Then
                                ' zulassen -> Startkategorie und Endkategorie sind identisch
                                '             Falls nur ein Knoten enthalten ist, resultiert ein geschlossener Pfad,
                                '             was dem Grundszenario mit "Heimknoten" ([Zuhause] HomeSweetHome) entspricht.
                            End If
                        Else
                            If CAT Is m_CATH Then
                                ' Einschrnkung (*)
                                RaiseError m_sFileTask & ": Startkategorie """ & Stris(i) & """ kann unterwegs nicht besucht werden."
                                Exit Function
                            End If
                        End If
                    Else
                        ' Einschrnkung (*)
                        RaiseError m_sFileTask & ": Mehrfaches Auftreten von Kategorie """ & Stris(i) & """"
                        Exit Function
                    End If
                End If
            Next
            InitTask = True
        Else
            RaiseError m_sFileTask & ": Mindestens 2 Teilstrecken erwartet"
        End If
    End If
    
    ' (*) Ohne diese Einschrnkung wren Node.m_NODPrev und Node.m_MinPath nicht eindeutig.
    '     Somit msste eine weitere Klasse [PathInfo] definiert werden,
    '     um die krzesten Pfade (1.Besuch, 2.Besuch etc.) festzuhalten
End Function
Public Sub RaiseError(ByVal Msg As String)
    MsgBox Msg, vbCritical, App.EXEName & " Error"
    ErrorRaised = True
End Sub
Public Function AddGlobNode(NOD As Node) As Boolean
    ' berprfen, ob NOD.Name global eindeutig ist
    Dim NOD2 As Node
    Set NOD2 = AllNodeItem(NOD.Name)
    If NOD2 Is Nothing Then
        m_AllNodeColl.Add NOD, NOD.Name
        AddGlobNode = True
    Else
        RaiseError "Node-Name """ & NOD.Name & """ nicht eindeutig:" & vbNewLine & _
                   "Tritt in den Kategorien [" & NOD2.CAT.Name & "] und [" & NOD.CAT.Name & "] auf"
    End If
End Function
Private Function AllNodeItem(ByVal NodeName As String) As Node
    On Error Resume Next
    Set AllNodeItem = m_AllNodeColl(NodeName)
End Function
Private Function CategoryItem(ByVal Name As String) As Category
    On Error Resume Next
    Set CategoryItem = m_CategoryColl(Name)
End Function
Private Function SortedCategoryItem(ByVal Name As String) As Category
    On Error Resume Next
    Set SortedCategoryItem = m_SortedCategoryColl(Name)
End Function
Public Property Get sFileTask() As String
    sFileTask = m_sFileTask
End Property
Public Property Get sFileStations() As String
    sFileStations = m_sFileStations
End Property
Public Property Get sFileOutput() As String
    sFileOutput = m_sFileOutput
End Property

